// create global logger
var l = new Logger("auto-off.log");

// This function simply saves the user defined auto-off parameters in the
// property tree.
//
// parameters.zoneID
// parameters.offDelay
function setTrigger(parameters) 
{
    var zoneID = parseInt(parameters.ZoneID, 10);
    var offDelay = parseInt(parameters.offDelay, 10);
    if (isNaN(zoneID))
    { 
        l.logln("Set trigger: invalid zoneID parameter " + parameters.zoneID);
        return;
    }

    if (isNaN(offDelay)) 
    {
        l.logln("Set trigger: invalid offDelay parameter " + 
                parameters.offDelay);
        return;
    }

    l.logln("Setting trigger for zone: " + zoneID + " with delay: " + offDelay);
    Property.setProperty('settings/' + zoneID + '/ZoneID', zoneID);
    Property.setFlag('settings/' + zoneID + '/ZoneID', 'ARCHIVE', true);
    Property.setProperty('settings/' + zoneID + '/offDelay', offDelay);
    Property.setFlag('settings/' + zoneID + '/offDelay', 'ARCHIVE', true);
    Property.setFlag('settings/' + zoneID, 'ARCHIVE', true);
    Property.setFlag('settings', 'ARCHIVE', true);
    Property.store();
}

// This function checks if all required conditions are met and issues
// a scene 0 (off) call on the specified zone.
function executeTrigger(zoneID, triggerID)
{
    l.logln("Executing tigger for zone " + zoneID);

    // our idea is to turn off the light only if scene 1 was called,
    // if the user switches to a different scene we will not do anything
    // and assume that auto-off is not wanted
    //
    // dSS saves the last called scene for each zone in the property tree, so
    // we will just read it from there
    var lastCalledScene = Property.getProperty('/apartment/zones/zone' +
                                            zoneID + 
                                            '/groups/group1/lastCalledScene');

    // only take action if we are still in scene 1
    if (lastCalledScene == Scene.User1)
    {
        // each time a trigger is queued by the processSceneCall function,
        // a new trigger id for the desired zone is generated and saved in the
        // property tree
        var expectedTriggerID = Property.getProperty('settings/' + zoneID +
                                                     '/triggerID');
        l.logln("Expecting trigger: "+ expectedTriggerID + " got " + triggerID);

        // if the id's do not match, then someone turned off the light before
        // the timeout elapsed and then turned the lights back on again. in the
        // latter case a new trigger was queued and this one is obsolete
        if (triggerID === expectedTriggerID)
        {
            l.logln("Trigger id matches, executing!");
            // select all "light" devices from the desired zone by creating
            // a set, then call scene 0 (off)
            var set = getDevices().byZone(zoneID).byGroup('yellow');
            set.callScene(Scene.Off);
            return;
        }
    
        l.logln("Trigger id does not match, returning");
    }
}

// This function is executed each time a scene is called. We will receive
// information in which zone the call happened and what scene was called
function processSceneCall(parameters)
{
    var sceneID = raisedEvent.parameter.sceneID;
    var zoneID = raisedEvent.source.zoneID;
    l.logln("Processing call scene event for zone " + zoneID + 
            " scene " + sceneID);

    if ((sceneID === undefined) || (zoneID === undefined)) 
    {
        l.logln("Returning... invalid scene or zone data");
        return;
    }

    // ignore scenes that we do not care about
    if (sceneID != Scene.User1) 
    {
        l.logln("Ignoring szene " + sceneID);
        return;
    }

    // check if auto-off is configured for this zone
    var zoneNode = Property.getNode('settings/' + zoneID);
    if (zoneNode === null) 
    {
        l.logln("Nothing configured, returning");
        return;
    }

    // check the delay node
    var delayNode = zoneNode.getChild('offDelay');
    if (delayNode === null) 
    {
        l.logln("Node for zone " + zoneID + " does not have a valid offDelay " +
                "parameter, skipping trigger!");
        return;
    }

    // check if off delay setting is available and valid
    var offDelay = delayNode.getValue();
    if ((offDelay === null) || isNaN(offDelay)) 
    {
        l.logln("Invalid offDelay value " + offDelay + ", skipping trigger!");
        return;
    }

    // an off delay of zero means - auto off is disabled
    if (offDelay < 1)
    {
        l.logln("Auto-off for zone " + zoneID + " is disabled");
        return;
    }

    // special case: user turns the light on, then turns it off and back on 
    // again. in this situation the trigger that was queued first becomes 
    // obsolete and only the second trigger should turn off the light
    //
    // we will help ourselves by creating an id for each trigger. we will
    // save it in the property tree before queueing the executeTrigger function
    // and we will pass it to the function as a parameter
    //
    // once the timeout elapses and the executeTrigger function gets called,
    // it will load the trigger id from the property tree and compare it
    // to the id that was passed as function parameter
    var triggerID = Date.parse(new Date()).toString();
    Property.setProperty('settings/' + zoneID + '/triggerID', triggerID);

    // our delay is in minutes, timeout parameter has to be in ms
    setTimeout(offDelay * 60 * 1000, 
               function() { executeTrigger(zoneID, triggerID); });
}

// we subscribe to three events: running, auto-off_set and callScene
//
// we receive the "running" event when the dSS starts up, we will use it to 
// load our property tree configuration
if (raisedEvent.name === 'running')
{
    Property.load();
}
// "auto-off_set" event is set by the web UI
else if (raisedEvent.name === 'auto-off_set') 
{
    setTrigger(raisedEvent.parameter);
}
// each time a scene is called, we will receive a "callScene" event
else if (raisedEvent.name === 'callScene') 
{
    processSceneCall(raisedEvent.parameter);
}
